@@ -12,6 +12,13 @@
  * $HEADER$
  */
 
+/**
+ * @file
+ *
+ * This files contains all the hierarchical implementations of allreduce
+ * Only work with regular situation (each node has equal number of processes)
+ */
+
 #include "coll_han.h"
 #include "ompi/mca/coll/base/coll_base_functions.h"
 #include "ompi/mca/coll/base/coll_tags.h"
@@ -172,16 +179,16 @@ mca_coll_han_allreduce_intra(const void *sbuf,
     issue_task(t3);
 
     while (t->completed[0] != t->num_segments) {
-        /* Create t3 tasks for the current segment */
-        mca_coll_task_t *t3 = OBJ_NEW(mca_coll_task_t);
-        /* Setup up t3 task arguments */
-        t->cur_task = t3;
-        t->sbuf = (char *) t->sbuf + extent * t->seg_count;
+        /* Create t_next_seg tasks for the current segment */
+        mca_coll_task_t *t_next_seg = OBJ_NEW(mca_coll_task_t);
+        /* Setup up t_next_seg task arguments */
+        t->cur_task = t_next_seg;
+        t->sbuf = (t->sbuf == MPI_IN_PLACE) ? MPI_IN_PLACE : (char *) t->sbuf + extent * t->seg_count;
         t->rbuf = (char *) t->rbuf + extent * t->seg_count;
         t->cur_seg = t->cur_seg + 1;
-        /* Init t3 task */
-        init_task(t3, mca_coll_han_allreduce_t3_task, (void *) t);
-        issue_task(t3);
+        /* Init t_next_seg task */
+        init_task(t_next_seg, mca_coll_han_allreduce_t3_task, (void *) t);
+        issue_task(t_next_seg);
     }
     free(t->completed);
     t->completed = NULL;
@@ -194,7 +201,7 @@ mca_coll_han_allreduce_intra(const void *sbuf,
                                           comm, han_module->previous_allreduce_module);
 }
 
-/* t0 task */
+/* t0 task that performs a local reduction */
 int mca_coll_han_allreduce_t0_task(void *task_args)
 {
     mca_coll_han_allreduce_args_t *t = (mca_coll_han_allreduce_args_t *) task_args;
@@ -224,7 +231,7 @@ int mca_coll_han_allreduce_t0_task(void *task_args)
     return OMPI_SUCCESS;
 }
 
-/* t1 task */
+/* t1 task that performs a ireduce on top communicator */
 int mca_coll_han_allreduce_t1_task(void *task_args)
 {
     mca_coll_han_allreduce_args_t *t = (mca_coll_han_allreduce_args_t *) task_args;
@@ -254,11 +261,26 @@ int mca_coll_han_allreduce_t1_task(void *task_args)
         if (t->cur_seg == t->num_segments - 2 && t->last_seg_count != t->seg_count) {
             tmp_count = t->last_seg_count;
         }
-        t->low_comm->c_coll->coll_reduce((char *) t->sbuf + extent * t->seg_count,
-                                         (char *) t->rbuf + extent * t->seg_count, tmp_count,
-                                         t->dtype, t->op, t->root_low_rank, t->low_comm,
-                                         t->low_comm->c_coll->coll_reduce_module);
 
+        if (t->sbuf == MPI_IN_PLACE) {
+            if (!t->noop) {
+                t->low_comm->c_coll->coll_reduce(MPI_IN_PLACE,
+                                                 (char *) t->rbuf + extent * t->seg_count, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+            } else {
+                t->low_comm->c_coll->coll_reduce((char *) t->rbuf + extent * t->seg_count,
+                                                 NULL, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+
+            }
+        } else {
+            t->low_comm->c_coll->coll_reduce((char *) t->sbuf + extent * t->seg_count,
+                                             (char *) t->rbuf + extent * t->seg_count, tmp_count,
+                                             t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                             t->low_comm->c_coll->coll_reduce_module);
+	}
     }
     if (!t->noop) {
         ompi_request_wait(&ireduce_req, MPI_STATUS_IGNORE);
@@ -313,10 +335,26 @@ int mca_coll_han_allreduce_t2_task(void *task_args)
         if (t->cur_seg == t->num_segments - 3 && t->last_seg_count != t->seg_count) {
             tmp_count = t->last_seg_count;
         }
-        t->low_comm->c_coll->coll_reduce((char *) t->sbuf + 2 * extent * t->seg_count,
-                                         (char *) t->rbuf + 2 * extent * t->seg_count, tmp_count,
-                                         t->dtype, t->op, t->root_low_rank, t->low_comm,
-                                         t->low_comm->c_coll->coll_reduce_module);
+
+	if (t->sbuf == MPI_IN_PLACE) {
+	    if (!t->noop) {
+                t->low_comm->c_coll->coll_reduce(MPI_IN_PLACE,
+                                                 (char *) t->rbuf + 2 * extent * t->seg_count, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+	    } else {
+                t->low_comm->c_coll->coll_reduce((char *) t->rbuf + 2 * extent * t->seg_count,
+                                                 NULL, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+
+	    }
+	} else {
+            t->low_comm->c_coll->coll_reduce((char *) t->sbuf + 2 * extent * t->seg_count,
+                                             (char *) t->rbuf + 2 * extent * t->seg_count, tmp_count,
+                                             t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                             t->low_comm->c_coll->coll_reduce_module);
+	}
     }
     if (!t->noop && req_count > 0) {
         ompi_request_wait_all(req_count, reqs, MPI_STATUSES_IGNORE);
@@ -326,7 +364,7 @@ int mca_coll_han_allreduce_t2_task(void *task_args)
     return OMPI_SUCCESS;
 }
 
-/* t3 task */
+/* t3 task that performs broadcasts */
 int mca_coll_han_allreduce_t3_task(void *task_args)
 {
     mca_coll_han_allreduce_args_t *t = (mca_coll_han_allreduce_args_t *) task_args;
@@ -346,7 +384,7 @@ int mca_coll_han_allreduce_t3_task(void *task_args)
             if (t->cur_seg == t->num_segments - 2 && t->last_seg_count != t->seg_count) {
                 tmp_count = t->last_seg_count;
             }
-            t->up_comm->c_coll->coll_ibcast((char *) t->rbuf + extent * t->seg_count, t->seg_count,
+            t->up_comm->c_coll->coll_ibcast((char *) t->rbuf + extent * t->seg_count, tmp_count,
                                             t->dtype, t->root_up_rank, t->up_comm, &(reqs[0]),
                                             t->up_comm->c_coll->coll_ibcast_module);
             req_count++;
@@ -377,13 +415,34 @@ int mca_coll_han_allreduce_t3_task(void *task_args)
         if (t->cur_seg == t->num_segments - 4 && t->last_seg_count != t->seg_count) {
             tmp_count = t->last_seg_count;
         }
-        t->low_comm->c_coll->coll_reduce((char *) t->sbuf + 3 * extent * t->seg_count,
-                                         (char *) t->rbuf + 3 * extent * t->seg_count, tmp_count,
-                                         t->dtype, t->op, t->root_low_rank, t->low_comm,
-                                         t->low_comm->c_coll->coll_reduce_module);
+
+        if (t->sbuf == MPI_IN_PLACE) {
+            if (!t->noop) {
+                t->low_comm->c_coll->coll_reduce(MPI_IN_PLACE,
+                                                 (char *) t->rbuf + 3 * extent * t->seg_count, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+	    } else {
+                t->low_comm->c_coll->coll_reduce((char *) t->rbuf + 3 * extent * t->seg_count,
+                                                 NULL, tmp_count,
+                                                 t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                                 t->low_comm->c_coll->coll_reduce_module);
+            }
+        } else {
+            t->low_comm->c_coll->coll_reduce((char *) t->sbuf + 3 * extent * t->seg_count,
+                                             (char *) t->rbuf + 3 * extent * t->seg_count, tmp_count,
+                                             t->dtype, t->op, t->root_low_rank, t->low_comm,
+                                             t->low_comm->c_coll->coll_reduce_module);
+        }
     }
     /* lb of cur_seg */
-    t->low_comm->c_coll->coll_bcast((char *) t->rbuf, t->seg_count, t->dtype, t->root_low_rank,
+    if (t->cur_seg == t->num_segments - 1 && t->last_seg_count != t->seg_count) {
+        tmp_count = t->last_seg_count;
+    } else {
+        tmp_count = t->seg_count;
+    }
+
+    t->low_comm->c_coll->coll_bcast((char *) t->rbuf, tmp_count, t->dtype, t->root_low_rank,
                                     t->low_comm, t->low_comm->c_coll->coll_bcast_module);
     if (!t->noop && req_count > 0) {
         ompi_request_wait_all(req_count, reqs, MPI_STATUSES_IGNORE);
@@ -397,6 +456,10 @@ int mca_coll_han_allreduce_t3_task(void *task_args)
     return OMPI_SUCCESS;
 }
 
+/*
+ * Short implementation of allreduce that only does hierarchical
+ * communications without tasks.
+ */
 int
 mca_coll_han_allreduce_intra_simple(const void *sbuf,
                                     void *rbuf,
