@@ -121,9 +121,9 @@ static int mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
 */
 static int ucc_comm_attr_del_fn(MPI_Comm comm, int keyval, void *attr_val, void *extra)
 {
-
     mca_coll_ucc_module_t *ucc_module = (mca_coll_ucc_module_t*) attr_val;
-    ucc_team_destroy(ucc_module->ucc_team);
+    ucc_status_t status;
+    while(UCC_INPROGRESS == (status = ucc_team_destroy(ucc_module->ucc_team))) {}
     if (ucc_module->comm == &ompi_mpi_comm_world.comm) {
         if (mca_coll_ucc_component.libucc_initialized) {
             UCC_VERBOSE(1,"finalizing ucc library");
@@ -132,6 +132,10 @@ static int ucc_comm_attr_del_fn(MPI_Comm comm, int keyval, void *attr_val, void
             ucc_finalize(mca_coll_ucc_component.ucc_lib);
         }
     }
+    if (UCC_OK != status) {
+        UCC_ERROR("UCC team destroy failed");
+        return OMPI_ERROR;
+    }
     return OMPI_SUCCESS;
 }
 
@@ -234,10 +238,12 @@ static int mca_coll_ucc_init_ctx() {
         UCC_ERROR("UCC lib config read failed");
         return OMPI_ERROR;
     }
-    if (UCC_OK != ucc_lib_config_modify(lib_config, "CLS", cm->cls)) {
-        ucc_lib_config_release(lib_config);
-        UCC_ERROR("failed to modify UCC lib config to set CLS");
-        return OMPI_ERROR;
+    if (strlen(cm->cls) > 0) {
+        if (UCC_OK != ucc_lib_config_modify(lib_config, "CLS", cm->cls)) {
+            ucc_lib_config_release(lib_config);
+            UCC_ERROR("failed to modify UCC lib config to set CLS");
+            return OMPI_ERROR;
+        }
     }
 
     if (UCC_OK != ucc_init(&lib_params, lib_config, &cm->ucc_lib)) {
@@ -509,7 +515,7 @@ int mca_coll_ucc_req_free(struct ompi_request_t **ompi_req)
 {
     opal_free_list_return (&mca_coll_ucc_component.requests,
                            (opal_free_list_item_t *)(*ompi_req));
-    *ompi_req = &ompi_request_empty;
+    *ompi_req = MPI_REQUEST_NULL;
     return OMPI_SUCCESS;
 }
 
