@@ -98,9 +98,6 @@ ompi_osc_portals4_module_t ompi_osc_portals4_module_template = {
         ompi_osc_portals4_flush_all,
         ompi_osc_portals4_flush_local,
         ompi_osc_portals4_flush_local_all,
-
-        ompi_osc_portals4_set_info,
-        ompi_osc_portals4_get_info
     }
 };
 
@@ -230,7 +227,7 @@ process:
             }
 
             req = (ompi_osc_portals4_request_t*) ev.user_ptr;
-            opal_atomic_add_fetch_size_t(&req->super.req_status._ucount, ev.mlength);
+            req->super.req_status._ucount = opal_atomic_add_fetch_32(&req->bytes_committed, ev.mlength);
             ops = opal_atomic_add_fetch_32(&req->ops_committed, 1);
             if (ops == req->ops_expected) {
                 ompi_request_complete(&req->super, true);
