@@ -76,6 +76,7 @@ int MPI_Unpack(const void *inbuf, int insize, int *position,
         OMPI_ERRHANDLER_CHECK(rc, comm, rc, FUNC_NAME);
     }
 
+    OPAL_CR_ENTER_LIBRARY();
 
     if( insize > 0 ) {
         int ret;
@@ -88,6 +89,7 @@ int MPI_Unpack(const void *inbuf, int insize, int *position,
         opal_convertor_get_packed_size( &local_convertor, &size );
         if( (*position + size) > (unsigned int)insize ) {
             OBJ_DESTRUCT( &local_convertor );
+            OPAL_CR_EXIT_LIBRARY();
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TRUNCATE, FUNC_NAME);
         }
 
@@ -107,5 +109,7 @@ int MPI_Unpack(const void *inbuf, int insize, int *position,
         }
     }
 
+    OPAL_CR_EXIT_LIBRARY();
+
     OMPI_ERRHANDLER_RETURN(rc, comm, MPI_ERR_UNKNOWN, FUNC_NAME);
 }
