@@ -2,6 +2,7 @@
  * Copyright (c) 2013      Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -15,6 +16,8 @@
 #include "ompi/runtime/params.h"
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/win/win.h"
+#include "ompi/communicator/communicator.h"
+#include "opal/util/info_subscriber.h"
 
 #if OMPI_BUILD_MPI_PROFILING
 #if OPAL_HAVE_WEAK_SYMBOLS
@@ -43,6 +46,9 @@ int MPI_Win_set_info(MPI_Win win, MPI_Info info)
         }
     }
 
-    ret = win->w_osc_module->osc_set_info(win, info);
+    OPAL_CR_ENTER_LIBRARY();
+
+    ret = opal_infosubscribe_change_info(&(win->super), &(info->super));
+
     OMPI_ERRHANDLER_RETURN(ret, win, ret, FUNC_NAME);
 }
