@@ -62,17 +62,39 @@ int opal_datatype_contain_basic_datatypes( const opal_datatype_t* pData, char* p
 int opal_datatype_dump_data_flags( unsigned short usflags, char* ptr, size_t length )
 {
     int index = 0;
-    if( length < 22 ) return 0;
-    index = snprintf( ptr, 22, "-----------[---][---]" );  /* set everything to - */
-    if( usflags & OPAL_DATATYPE_FLAG_COMMITTED )  ptr[1]  = 'c';
-    if( usflags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) ptr[2]  = 'C';
-    if( usflags & OPAL_DATATYPE_FLAG_OVERLAP )    ptr[3]  = 'o';
-    if( usflags & OPAL_DATATYPE_FLAG_USER_LB )    ptr[4]  = 'l';
-    if( usflags & OPAL_DATATYPE_FLAG_USER_UB )    ptr[5]  = 'u';
-    if( usflags & OPAL_DATATYPE_FLAG_PREDEFINED ) ptr[6]  = 'P';
-    if( !(usflags & OPAL_DATATYPE_FLAG_NO_GAPS) ) ptr[7]  = 'G';
-    if( usflags & OPAL_DATATYPE_FLAG_DATA )       ptr[8]  = 'D';
-    if( (usflags & OPAL_DATATYPE_FLAG_BASIC) == OPAL_DATATYPE_FLAG_BASIC ) ptr[9]  = 'B';
+    if (length < 22) {
+        return 0;
+    }
+    index = snprintf(ptr, 22, "-----------[---][---]"); /* set everything to - */
+    if (usflags & OPAL_DATATYPE_FLAG_COMMITTED) {
+        ptr[1] = 'c';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_CONTIGUOUS) {
+        ptr[2] = 'C';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_OVERLAP) {
+        ptr[3] = 'o';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_USER_LB) {
+        ptr[4] = 'l';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_USER_UB) {
+        ptr[5] = 'u';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_PREDEFINED) {
+        ptr[6] = 'P';
+    }
+    if (!(usflags & OPAL_DATATYPE_FLAG_NO_GAPS)) {
+        ptr[7] = 'G';
+    }
+    if (usflags & OPAL_DATATYPE_FLAG_DATA) {
+        ptr[8] = 'D';
+    }
+    if ((usflags & OPAL_DATATYPE_FLAG_BASIC) == OPAL_DATATYPE_FLAG_BASIC) {
+        ptr[9] = 'B';
+    } else if (usflags & OPAL_DATATYPE_OPTIMIZED_RESTRICTED) {
+        ptr[9] = 'H';  /* optimized description restricted to homogeneous cases */
+    }
     /* We know nothing about the upper level language or flags! */
     /* ... */
     return index;
