@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -12,6 +13,8 @@
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserverd.
  * Copyright (c) 2016      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -38,9 +41,9 @@
  *********************************************************************/
 #define OPAL_HAVE_ATOMIC_MEM_BARRIER 1
 
-#define OPAL_HAVE_ATOMIC_CMPSET_32 1
+#define OPAL_HAVE_ATOMIC_COMPARE_EXCHANGE_32 1
 
-#define OPAL_HAVE_ATOMIC_CMPSET_64 1
+#define OPAL_HAVE_ATOMIC_COMPARE_EXCHANGE_64 1
 
 
 /**********************************************************************
@@ -82,50 +85,49 @@ static inline void opal_atomic_isync(void)
  *********************************************************************/
 #if OPAL_GCC_INLINE_ASSEMBLY
 
-static inline int opal_atomic_cmpset_32( volatile int32_t *addr,
-                                         int32_t oldval, int32_t newval)
+static inline bool opal_atomic_compare_exchange_strong_32 (volatile int32_t *addr, int32_t *oldval, int32_t newval)
 {
-   /* casa [reg(rs1)] %asi, reg(rs2), reg(rd)
-    *
-    * if (*(reg(rs1)) == reg(rs2) )
-    *    swap reg(rd), *(reg(rs1))
-    * else
-    *    reg(rd) = *(reg(rs1))
-    */
+    /* casa [reg(rs1)] %asi, reg(rs2), reg(rd)
+     *
+     * if (*(reg(rs1)) == reg(rs2) )
+     *    swap reg(rd), *(reg(rs1))
+     * else
+     *    reg(rd) = *(reg(rs1))
+     */
 
-   int32_t ret = newval;
+    int32_t prev = newval;
+    bool ret;
 
-   __asm__ __volatile__("casa [%1] " ASI_P ", %2, %0"
-                      : "+r" (ret)
-                      : "r" (addr), "r" (oldval));
-   return (ret == oldval);
+    __asm__ __volatile__("casa [%1] " ASI_P ", %2, %0"
+                         : "+r" (prev)
+                         : "r" (addr), "r" (*oldval));
+    ret = (prev == *oldval);
+    *oldval = prev;
+    return ret;
 }
 
 
-static inline int opal_atomic_cmpset_acq_32( volatile int32_t *addr,
-                                             int32_t oldval, int32_t newval)
+static inline bool opal_atomic_compare_exchange_strong_acq_32 (volatile int32_t *addr, int32_t *oldval, int32_t newval)
 {
-   int rc;
+    bool rc;
 
-   rc = opal_atomic_cmpset_32(addr, oldval, newval);
-   opal_atomic_rmb();
+    rc = opal_atomic_compare_exchange_strong_32 (addr, oldval, newval);
+    opal_atomic_rmb();
 
-   return rc;
+    return rc;
 }
 
 
-static inline int opal_atomic_cmpset_rel_32( volatile int32_t *addr,
-                                             int32_t oldval, int32_t newval)
+static inline bool opal_atomic_compare_exchange_strong_rel_32 (volatile int32_t *addr, int32_t *oldval, int32_t newval)
 {
-   opal_atomic_wmb();
-   return opal_atomic_cmpset_32(addr, oldval, newval);
+    opal_atomic_wmb();
+    return opal_atomic_compare_exchange_strong_32 (addr, oldval, newval);
 }
 
 
 #if OPAL_ASSEMBLY_ARCH == OPAL_SPARCV9_64
 
-static inline int opal_atomic_cmpset_64( volatile int64_t *addr,
-                                         int64_t oldval, int64_t newval)
+static inline bool opal_atomic_compare_exchange_strong_64 (volatile int64_t *addr, int64_t *oldval, int64_t newval)
 {
     /* casa [reg(rs1)] %asi, reg(rs2), reg(rd)
      *
@@ -134,18 +136,20 @@ static inline int opal_atomic_cmpset_64( volatile int64_t *addr,
      * else
      *    reg(rd) = *(reg(rs1))
      */
-   int64_t ret = newval;
+    int64_t prev = newval;
+    bool ret;
 
-   __asm__ __volatile__("casxa [%1] " ASI_P ", %2, %0"
-                      : "+r" (ret)
-                      : "r" (addr), "r" (oldval));
-   return (ret == oldval);
+    __asm__ __volatile__("casxa [%1] " ASI_P ", %2, %0"
+                         : "+r" (prev)
+                         : "r" (addr), "r" (*oldval));
+    ret = (prev == *oldval);
+    *oldval = prev;
+    return ret;
 }
 
 #else /* OPAL_ASSEMBLY_ARCH == OPAL_SPARCV9_64 */
 
-static inline int opal_atomic_cmpset_64( volatile int64_t *addr,
-                                         int64_t oldval, int64_t newval)
+static inline bool opal_atomic_compare_exchange_strong_64 (volatile int64_t *addr, int64_t *oldval, int64_t newval)
 {
     /* casa [reg(rs1)] %asi, reg(rs2), reg(rd)
      *
@@ -155,40 +159,41 @@ static inline int opal_atomic_cmpset_64( volatile int64_t *addr,
      *    reg(rd) = *(reg(rs1))
      *
      */
-    long long ret = newval;
+    int64_t prev = newval;
+    bool ret;
 
     __asm__ __volatile__(
                        "ldx %0, %%g1               \n\t" /* g1 = ret */
                        "ldx %2, %%g2               \n\t" /* g2 = oldval */
                        "casxa [%1] " ASI_P ", %%g2, %%g1 \n\t"
                        "stx %%g1, %0               \n"
-                       : "+m"(ret)
-                       : "r"(addr), "m"(oldval)
+                       : "+m"(prev)
+                       : "r"(addr), "m"(*oldval)
                        : "%g1", "%g2"
                        );
 
-   return (ret == oldval);
+   ret = (prev == *oldval);
+   *oldval = prev;
+   return ret;
 }
 
 #endif /* OPAL_ASSEMBLY_ARCH == OPAL_SPARCV9_64 */
 
-static inline int opal_atomic_cmpset_acq_64( volatile int64_t *addr,
-                                             int64_t oldval, int64_t newval)
+static inline bool opal_atomic_compare_exchange_strong_acq_64 (volatile int64_t *addr, int64_t *oldval, int64_t newval)
 {
-   int rc;
+    bool rc;
 
-   rc = opal_atomic_cmpset_64(addr, oldval, newval);
-   opal_atomic_rmb();
+    rc = opal_atomic_compare_exchange_strong_64 (addr, oldval, newval);
+    opal_atomic_rmb();
 
-   return rc;
+    return rc;
 }
 
 
-static inline int opal_atomic_cmpset_rel_64( volatile int64_t *addr,
-                                             int64_t oldval, int64_t newval)
+static inline bool opal_atomic_compare_exchange_strong_rel_64 (volatile int64_t *addr, int64_t *oldval, int64_t newval)
 {
-   opal_atomic_wmb();
-   return opal_atomic_cmpset_64(addr, oldval, newval);
+    opal_atomic_wmb();
+    return opal_atomic_compare_exchange_strong_64 (addr, oldval, newval);
 }
 
 #endif /* OPAL_GCC_INLINE_ASSEMBLY */
