@@ -269,8 +269,8 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
     fifo->queue = (volatile void **) VIRTUAL2RELATIVE(fifo->queue_recv);
 
     /* initialize the locks */
-    opal_atomic_init(&(fifo->head_lock), OPAL_ATOMIC_UNLOCKED);
-    opal_atomic_init(&(fifo->tail_lock), OPAL_ATOMIC_UNLOCKED);
+    opal_atomic_lock_init(&(fifo->head_lock), OPAL_ATOMIC_LOCK_UNLOCKED);
+    opal_atomic_lock_init(&(fifo->tail_lock), OPAL_ATOMIC_LOCK_UNLOCKED);
     opal_atomic_unlock(&(fifo->head_lock));  /* should be unnecessary */
     opal_atomic_unlock(&(fifo->tail_lock));  /* should be unnecessary */
 
@@ -517,6 +517,13 @@ extern void mca_btl_smcuda_dump(struct mca_btl_base_module_t* btl,
                             struct mca_btl_base_endpoint_t* endpoint,
                             int verbose);
 
+/**
+ * Fault Tolerance Event Notification Function
+ * @param state Checkpoint Stae
+ * @return OPAL_SUCCESS or failure status
+ */
+int mca_btl_smcuda_ft_event(int state);
+
 #if OPAL_ENABLE_PROGRESS_THREADS == 1
 void mca_btl_smcuda_component_event_thread(opal_object_t*);
 #endif
