@@ -22,6 +22,7 @@
  * Copyright (c) 2019      IBM Corporation. All rights reserved.
  * Copyright (c) 2019-2020 Inria.  All rights reserved.
  * Copyright (c) 2020      Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -373,13 +374,15 @@ int opal_hwloc_base_get_topology(void)
                             "hwloc:base loading topology from XML");
         /* load the topology */
         if (0 != hwloc_topology_init(&opal_hwloc_topology)) {
+            /* we can't recover from this error */
             free(val);
             return OPAL_ERROR;
         }
         if (0 != hwloc_topology_set_xmlbuffer(opal_hwloc_topology, val, strlen(val)+1)) {
+            /* default to discovery */
             free(val);
             hwloc_topology_destroy(opal_hwloc_topology);
-            return OPAL_ERROR;
+            goto discover;
         }
         /* since we are loading this from an external source, we have to
          * explicitly set a flag so hwloc sets things up correctly
@@ -387,15 +390,17 @@ int opal_hwloc_base_get_topology(void)
         if (0 != opal_hwloc_base_topology_set_flags(opal_hwloc_topology,
                                                     HWLOC_TOPOLOGY_FLAG_IS_THISSYSTEM,
                                                     true)) {
+            /* default to discovery */
             hwloc_topology_destroy(opal_hwloc_topology);
             free(val);
-            return OPAL_ERROR;
+            goto discover;
         }
         /* now load the topology */
         if (0 != hwloc_topology_load(opal_hwloc_topology)) {
+            /* default to discovery */
             hwloc_topology_destroy(opal_hwloc_topology);
             free(val);
-            return OPAL_ERROR;
+            goto discover;
         }
         free(val);
         /* filter the cpus thru any default cpu set */
@@ -404,6 +409,7 @@ int opal_hwloc_base_get_topology(void)
             return rc;
         }
     } else if (NULL == opal_hwloc_base_topo_file) {
+    discover:
         opal_output_verbose(1, opal_hwloc_base_framework.framework_output,
                             "hwloc:base discovering topology");
         if (0 != hwloc_topology_init(&opal_hwloc_topology) ||
