@@ -298,7 +298,7 @@ char *opal_path_access(char *fname, char *path, int mode)
  *  Loads argument array with $PATH env var.
  *
  *  Accepts
- *      -path String contiaing the $PATH
+ *      -path String containing the $PATH
  *      -argc Pointer to argc
  *      -argv Pointer to list of argv
  */
@@ -704,7 +704,7 @@ opal_path_df(const char *path,
 
     /* now set the amount of free space available on path */
                                /* sometimes buf.f_bavail is negative */
-    *out_avail = (uint64_t)buf.f_bsize * (uint64_t)(buf.f_bavail < 0 ? 0 : buf.f_bavail);
+    *out_avail = (uint64_t)buf.f_bsize * (uint64_t)((long)buf.f_bavail < 0 ? 0 : buf.f_bavail);
 
     OPAL_OUTPUT_VERBOSE((10, 2, "opal_path_df: stat(v)fs states "
                          "path: %s has %"PRIu64 " B of free space.",
