@@ -11,10 +11,12 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011-2015 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2014      Research Organization for Information Science
+ * Copyright (c) 2014-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2017      Mellanox Technologies. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -46,17 +48,16 @@
 #include "orte/mca/ess/ess.h"
 #include "orte/mca/rml/rml.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/threads.h"
 #include "orte/mca/odls/odls_types.h"
 
 #include "orte/mca/iof/base/base.h"
+#include "orte/mca/iof/base/iof_base_setup.h"
 #include "iof_hnp.h"
 
 /* LOCAL FUNCTIONS */
 static void stdin_write_handler(int fd, short event, void *cbdata);
 
-static void
-orte_iof_hnp_exception_handler(orte_process_name_t* peer, orte_rml_exception_t reason);
-
 /* API FUNCTIONS */
 static int init(void);
 
@@ -69,8 +70,16 @@ static int hnp_pull(const orte_process_name_t* src_name,
 static int hnp_close(const orte_process_name_t* peer,
                  orte_iof_tag_t source_tag);
 
+static int hnp_output(const orte_process_name_t* peer,
+                      orte_iof_tag_t source_tag,
+                      const char *msg);
+
+static void hnp_complete(const orte_job_t *jdata);
+
 static int finalize(void);
 
+static int hnp_ft_event(int state);
+
 /* The API's in this module are solely used to support LOCAL
  * procs - i.e., procs that are co-located to the HNP. Remote
  * procs interact with the HNP's IOF via the HNP's receive function,
@@ -78,20 +87,19 @@ static int finalize(void);
  */
 
 orte_iof_base_module_t orte_iof_hnp_module = {
-    init,
-    hnp_push,
-    hnp_pull,
-    hnp_close,
-    NULL,
-    finalize,
-    NULL
+    .init = init,
+    .push = hnp_push,
+    .pull = hnp_pull,
+    .close = hnp_close,
+    .output = hnp_output,
+    .complete = hnp_complete,
+    .finalize = finalize,
+    .ft_event = hnp_ft_event
 };
 
 /* Initialize the module */
 static int init(void)
 {
-    int rc;
-
     /* post non-blocking recv to catch forwarded IO from
      * the orteds
      */
@@ -101,13 +109,6 @@ static int init(void)
                             orte_iof_hnp_recv,
                             NULL);
 
-    if (ORTE_SUCCESS != (rc = orte_rml.add_exception_handler(orte_iof_hnp_exception_handler))) {
-        ORTE_ERROR_LOG(rc);
-        orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_IOF_HNP);
-        return rc;
-    }
-
-    OBJ_CONSTRUCT(&mca_iof_hnp_component.sinks, opal_list_t);
     OBJ_CONSTRUCT(&mca_iof_hnp_component.procs, opal_list_t);
     mca_iof_hnp_component.stdinev = NULL;
 
@@ -134,14 +135,9 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
 {
     orte_job_t *jdata;
     orte_proc_t *proc;
-    orte_iof_sink_t *sink;
-    orte_iof_proc_t *proct;
-    opal_list_item_t *item;
-    int flags;
-    char *outfile;
-    int fdout;
-    int np, numdigs;
-    orte_ns_cmp_bitmask_t mask;
+    orte_iof_proc_t *proct, *pptr;
+    int flags, rc;
+    orte_ns_cmp_bitmask_t mask = ORTE_NS_CMP_ALL;
 
     /* don't do this if the dst vpid is invalid or the fd is negative! */
     if (ORTE_VPID_INVALID == dst_name->vpid || fd < 0) {
@@ -153,6 +149,20 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          fd, ORTE_NAME_PRINT(dst_name)));
 
+    /* do we already have this process in our list? */
+    OPAL_LIST_FOREACH(proct, &mca_iof_hnp_component.procs, orte_iof_proc_t) {
+        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proct->name, dst_name)) {
+            /* found it */
+            goto SETUP;
+        }
+    }
+    /* if we get here, then we don't yet have this proc in our list */
+    proct = OBJ_NEW(orte_iof_proc_t);
+    proct->name.jobid = dst_name->jobid;
+    proct->name.vpid = dst_name->vpid;
+    opal_list_append(&mca_iof_hnp_component.procs, &proct->super);
+
+  SETUP:
     if (!(src_tag & ORTE_IOF_STDIN)) {
         /* set the file descriptor to non-blocking - do this before we setup
          * and activate the read event in case it fires right away
@@ -164,79 +174,63 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
             flags |= O_NONBLOCK;
             fcntl(fd, F_SETFL, flags);
         }
-        /* do we already have this process in our list? */
-        for (item = opal_list_get_first(&mca_iof_hnp_component.procs);
-             item != opal_list_get_end(&mca_iof_hnp_component.procs);
-             item = opal_list_get_next(item)) {
-            proct = (orte_iof_proc_t*)item;
-            mask = ORTE_NS_CMP_ALL;
-            if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proct->name, dst_name)) {
-                /* found it */
-                goto SETUP;
-            }
+        /* get the local jobdata for this proc */
+        if (NULL == (jdata = orte_get_job_data_object(proct->name.jobid))) {
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+            return ORTE_ERR_NOT_FOUND;
         }
-        /* if we get here, then we don't yet have this proc in our list */
-        proct = OBJ_NEW(orte_iof_proc_t);
-        proct->name.jobid = dst_name->jobid;
-        proct->name.vpid = dst_name->vpid;
-        opal_list_append(&mca_iof_hnp_component.procs, &proct->super);
-        /* see if we are to output to a file */
-        if (NULL != orte_output_filename) {
-            /* get the jobdata for this proc */
-            if (NULL == (jdata = orte_get_job_data_object(dst_name->jobid))) {
-                ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-                return ORTE_ERR_NOT_FOUND;
-            }
-            np = jdata->num_procs / 10;
-            /* determine the number of digits required for max vpid */
-            numdigs = 1;
-            while (np > 0) {
-                numdigs++;
-                np = np / 10;
-            }
-            /* construct the filename */
-            asprintf(&outfile, "%s.%d.%0*lu", orte_output_filename,
-                     (int)ORTE_LOCAL_JOBID(proct->name.jobid),
-                     numdigs, (unsigned long)proct->name.vpid);
-            /* create the file */
-            fdout = open(outfile, O_CREAT|O_RDWR|O_TRUNC, 0644);
-            free(outfile);
-            if (fdout < 0) {
-                /* couldn't be opened */
-                ORTE_ERROR_LOG(ORTE_ERR_FILE_OPEN_FAILURE);
-                return ORTE_ERR_FILE_OPEN_FAILURE;
-            }
-            /* define a sink to that file descriptor */
-            ORTE_IOF_SINK_DEFINE(&sink, dst_name, fdout, ORTE_IOF_STDOUTALL,
-                                 orte_iof_base_write_handler,
-                                 &mca_iof_hnp_component.sinks);
-        }
-
-    SETUP:
         /* define a read event and activate it */
         if (src_tag & ORTE_IOF_STDOUT) {
-            ORTE_IOF_READ_EVENT(&proct->revstdout, dst_name, fd, ORTE_IOF_STDOUT,
+            ORTE_IOF_READ_EVENT(&proct->revstdout, proct, fd, ORTE_IOF_STDOUT,
                                 orte_iof_hnp_read_local_handler, false);
         } else if (src_tag & ORTE_IOF_STDERR) {
-            ORTE_IOF_READ_EVENT(&proct->revstderr, dst_name, fd, ORTE_IOF_STDERR,
+            ORTE_IOF_READ_EVENT(&proct->revstderr, proct, fd, ORTE_IOF_STDERR,
                                 orte_iof_hnp_read_local_handler, false);
+#if OPAL_PMIX_V1
         } else if (src_tag & ORTE_IOF_STDDIAG) {
-            ORTE_IOF_READ_EVENT(&proct->revstddiag, dst_name, fd, ORTE_IOF_STDDIAG,
+            ORTE_IOF_READ_EVENT(&proct->revstddiag, proct, fd, ORTE_IOF_STDDIAG,
                                 orte_iof_hnp_read_local_handler, false);
+#endif
         }
+        /* setup any requested output files */
+        if (ORTE_SUCCESS != (rc = orte_iof_base_setup_output_files(dst_name, jdata, proct))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+
         /* if -all- of the readevents for this proc have been defined, then
          * activate them. Otherwise, we can think that the proc is complete
          * because one of the readevents fires -prior- to all of them having
          * been defined!
          */
-        if (NULL != proct->revstdout && NULL != proct->revstderr && NULL != proct->revstddiag) {
-            proct->revstdout->active = true;
-            opal_event_add(proct->revstdout->ev, 0);
-            proct->revstderr->active = true;
-            opal_event_add(proct->revstderr->ev, 0);
-            proct->revstddiag->active = true;
-            opal_event_add(proct->revstddiag->ev, 0);
-        }
+        if (NULL != proct->revstdout &&
+#if OPAL_PMIX_V1
+           NULL != proct->revstddiag &&
+#endif
+            (orte_iof_base.redirect_app_stderr_to_stdout || NULL != proct->revstderr)) {
+            if (proct->copy) {
+                /* see if there are any wildcard subscribers out there that
+                 * apply to us */
+                OPAL_LIST_FOREACH(pptr, &mca_iof_hnp_component.procs, orte_iof_proc_t) {
+                    if (dst_name->jobid == pptr->name.jobid &&
+                        ORTE_VPID_WILDCARD == pptr->name.vpid &&
+                        NULL != pptr->subscribers) {
+                        OBJ_RETAIN(pptr->subscribers);
+                        proct->subscribers = pptr->subscribers;
+                        break;
+                    }
+                }
+            }
+            ORTE_IOF_READ_ACTIVATE(proct->revstdout);
+            if (!orte_iof_base.redirect_app_stderr_to_stdout) {
+                ORTE_IOF_READ_ACTIVATE(proct->revstderr);
+            }
+#if OPAL_PMIX_V1
+            if (NULL != proct->revstddiag) {
+                ORTE_IOF_READ_ACTIVATE(proct->revstddiag);
+            }
+#endif
+       }
         return ORTE_SUCCESS;
     }
 
@@ -245,11 +239,10 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
      */
     if (ORTE_VPID_WILDCARD == dst_name->vpid) {
         /* if wildcard, define a sink with that info so it gets sent out */
-        ORTE_IOF_SINK_DEFINE(&sink, dst_name, -1, ORTE_IOF_STDIN,
-                             stdin_write_handler,
-                             &mca_iof_hnp_component.sinks);
-        sink->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
-        sink->daemon.vpid = ORTE_VPID_WILDCARD;
+        ORTE_IOF_SINK_DEFINE(&proct->stdinev, dst_name, -1, ORTE_IOF_STDIN,
+                             stdin_write_handler);
+        proct->stdinev->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
+        proct->stdinev->daemon.vpid = ORTE_VPID_WILDCARD;
      } else {
         /* no - lookup the proc's daemon and set that into sink */
         if (NULL == (jdata = orte_get_job_data_object(dst_name->jobid))) {
@@ -262,11 +255,10 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
         }
         /* if it is me, then don't set this up - we'll get it on the pull */
         if (ORTE_PROC_MY_NAME->vpid != proc->node->daemon->name.vpid) {
-            ORTE_IOF_SINK_DEFINE(&sink, dst_name, -1, ORTE_IOF_STDIN,
-                                 stdin_write_handler,
-                                 &mca_iof_hnp_component.sinks);
-            sink->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
-            sink->daemon.vpid = proc->node->daemon->name.vpid;
+            ORTE_IOF_SINK_DEFINE(&proct->stdinev, dst_name, -1, ORTE_IOF_STDIN,
+                                 stdin_write_handler);
+            proct->stdinev->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
+            proct->stdinev->daemon.vpid = proc->node->daemon->name.vpid;
         }
     }
 
@@ -309,7 +301,7 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
              * be dropped upon receipt at the local daemon
              */
             ORTE_IOF_READ_EVENT(&mca_iof_hnp_component.stdinev,
-                                dst_name, fd, ORTE_IOF_STDIN,
+                                proct, fd, ORTE_IOF_STDIN,
                                 orte_iof_hnp_read_local_handler, false);
 
             /* check to see if we want the stdin read event to be
@@ -317,15 +309,14 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
              * but may delay its activation
              */
             if (!(src_tag & ORTE_IOF_STDIN) || orte_iof_hnp_stdin_check(fd)) {
-                mca_iof_hnp_component.stdinev->active = true;
-                opal_event_add(mca_iof_hnp_component.stdinev->ev, 0);
+                ORTE_IOF_READ_ACTIVATE(mca_iof_hnp_component.stdinev);
             }
         } else {
             /* if we are not looking at a tty, just setup a read event
              * and activate it
              */
             ORTE_IOF_READ_EVENT(&mca_iof_hnp_component.stdinev,
-                                dst_name, fd, ORTE_IOF_STDIN,
+                                proct, fd, ORTE_IOF_STDIN,
                                 orte_iof_hnp_read_local_handler, true);
         }
     }
@@ -342,7 +333,8 @@ static int hnp_pull(const orte_process_name_t* dst_name,
                     orte_iof_tag_t src_tag,
                     int fd)
 {
-    orte_iof_sink_t *sink;
+    orte_iof_proc_t *proct;
+    orte_ns_cmp_bitmask_t mask = ORTE_NS_CMP_ALL;
     int flags;
 
     /* this is a local call - only stdin is supported */
@@ -366,11 +358,24 @@ static int hnp_pull(const orte_process_name_t* dst_name,
         fcntl(fd, F_SETFL, flags);
     }
 
-    ORTE_IOF_SINK_DEFINE(&sink, dst_name, fd, ORTE_IOF_STDIN,
-                         stdin_write_handler,
-                         &mca_iof_hnp_component.sinks);
-    sink->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
-    sink->daemon.vpid = ORTE_PROC_MY_NAME->vpid;
+    /* do we already have this process in our list? */
+    OPAL_LIST_FOREACH(proct, &mca_iof_hnp_component.procs, orte_iof_proc_t) {
+        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proct->name, dst_name)) {
+            /* found it */
+            goto SETUP;
+        }
+    }
+    /* if we get here, then we don't yet have this proc in our list */
+    proct = OBJ_NEW(orte_iof_proc_t);
+    proct->name.jobid = dst_name->jobid;
+    proct->name.vpid = dst_name->vpid;
+    opal_list_append(&mca_iof_hnp_component.procs, &proct->super);
+
+  SETUP:
+    ORTE_IOF_SINK_DEFINE(&proct->stdinev, dst_name, fd, ORTE_IOF_STDIN,
+                         stdin_write_handler);
+    proct->stdinev->daemon.jobid = ORTE_PROC_MY_NAME->jobid;
+    proct->stdinev->daemon.vpid = ORTE_PROC_MY_NAME->vpid;
 
     return ORTE_SUCCESS;
 }
@@ -382,48 +387,84 @@ static int hnp_pull(const orte_process_name_t* dst_name,
 static int hnp_close(const orte_process_name_t* peer,
                      orte_iof_tag_t source_tag)
 {
-    opal_list_item_t *item, *next_item;
-    orte_iof_sink_t* sink;
-    orte_ns_cmp_bitmask_t mask;
+    orte_iof_proc_t* proct;
+    orte_ns_cmp_bitmask_t mask = ORTE_NS_CMP_ALL;
 
-    for(item = opal_list_get_first(&mca_iof_hnp_component.sinks);
-        item != opal_list_get_end(&mca_iof_hnp_component.sinks);
-        item = next_item ) {
-        sink = (orte_iof_sink_t*)item;
-        next_item = opal_list_get_next(item);
-
-        mask = ORTE_NS_CMP_ALL;
-
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &sink->name, peer) &&
-           (source_tag & sink->tag)) {
-
-            /* No need to delete the event or close the file
-             * descriptor - the destructor will automatically
-             * do it for us.
-             */
-            opal_list_remove_item(&mca_iof_hnp_component.sinks, item);
-            OBJ_RELEASE(item);
+    OPAL_LIST_FOREACH(proct, &mca_iof_hnp_component.procs, orte_iof_proc_t) {
+        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proct->name, peer)) {
+            if (ORTE_IOF_STDIN & source_tag) {
+                if (NULL != proct->stdinev) {
+                    OBJ_RELEASE(proct->stdinev);
+                }
+                proct->stdinev = NULL;
+            }
+            if ((ORTE_IOF_STDOUT & source_tag) ||
+                (ORTE_IOF_STDMERGE & source_tag)) {
+                if (NULL != proct->revstdout) {
+                    orte_iof_base_static_dump_output(proct->revstdout);
+                    OBJ_RELEASE(proct->revstdout);
+                }
+                proct->revstdout = NULL;
+            }
+            if (ORTE_IOF_STDERR & source_tag) {
+                if (NULL != proct->revstderr) {
+                    orte_iof_base_static_dump_output(proct->revstderr);
+                    OBJ_RELEASE(proct->revstderr);
+                }
+                proct->revstderr = NULL;
+            }
+#if OPAL_PMIX_V1
+            if (ORTE_IOF_STDDIAG & source_tag) {
+                if (NULL != proct->revstddiag) {
+                    orte_iof_base_static_dump_output(proct->revstddiag);
+                    OBJ_RELEASE(proct->revstddiag);
+                }
+                proct->revstddiag = NULL;
+            }
+#endif
+            /* if we closed them all, then remove this proc */
+            if (NULL == proct->stdinev &&
+                NULL == proct->revstdout &&
+#if OPAL_PMIX_V1
+                NULL == proct->revstddiag &&
+#endif
+                NULL == proct->revstderr) {
+                opal_list_remove_item(&mca_iof_hnp_component.procs, &proct->super);
+                OBJ_RELEASE(proct);
+            }
             break;
         }
     }
     return ORTE_SUCCESS;
 }
 
+static void hnp_complete(const orte_job_t *jdata)
+{
+    orte_iof_proc_t *proct, *next;
+
+    /* cleanout any lingering sinks */
+    OPAL_LIST_FOREACH_SAFE(proct, next, &mca_iof_hnp_component.procs, orte_iof_proc_t) {
+        if (jdata->jobid == proct->name.jobid) {
+            opal_list_remove_item(&mca_iof_hnp_component.procs, &proct->super);
+            OBJ_RELEASE(proct);
+        }
+    }
+}
+
 static int finalize(void)
 {
-    opal_list_item_t* item;
-    orte_iof_write_output_t *output;
     orte_iof_write_event_t *wev;
-    int num_written;
+    orte_iof_proc_t *proct;
     bool dump;
+    orte_iof_write_output_t *output;
+    int num_written;
 
     /* check if anything is still trying to be written out */
     wev = orte_iof_base.iof_write_stdout->wev;
     if (!opal_list_is_empty(&wev->outputs)) {
         dump = false;
         /* make one last attempt to write this out */
-        while (NULL != (item = opal_list_remove_first(&wev->outputs))) {
-            output = (orte_iof_write_output_t*)item;
+        while (NULL != (output = (orte_iof_write_output_t*)opal_list_remove_first(&wev->outputs))) {
             if (!dump) {
                 num_written = write(wev->fd, output->data, output->numbytes);
                 if (num_written < output->numbytes) {
@@ -440,8 +481,7 @@ static int finalize(void)
         if (!opal_list_is_empty(&wev->outputs)) {
             dump = false;
             /* make one last attempt to write this out */
-            while (NULL != (item = opal_list_remove_first(&wev->outputs))) {
-                output = (orte_iof_write_output_t*)item;
+            while (NULL != (output = (orte_iof_write_output_t*)opal_list_remove_first(&wev->outputs))) {
                 if (!dump) {
                     num_written = write(wev->fd, output->data, output->numbytes);
                     if (num_written < output->numbytes) {
@@ -454,11 +494,35 @@ static int finalize(void)
         }
     }
 
-    orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_IOF_HNP);
+    /* cycle thru the procs and ensure all their output was delivered
+     * if they were writing to files */
+    while (NULL != (proct = (orte_iof_proc_t*)opal_list_remove_first(&mca_iof_hnp_component.procs))) {
+        if (NULL != proct->revstdout) {
+            orte_iof_base_static_dump_output(proct->revstdout);
+        }
+        if (NULL != proct->revstderr) {
+            orte_iof_base_static_dump_output(proct->revstderr);
+        }
+#if OPAL_PMIX_V1
+        if (NULL != proct->revstddiag) {
+            orte_iof_base_static_dump_output(proct->revstddiag);
+        }
+#endif
+        OBJ_RELEASE(proct);
+    }
+    OBJ_DESTRUCT(&mca_iof_hnp_component.procs);
 
     return ORTE_SUCCESS;
 }
 
+int hnp_ft_event(int state) {
+    /*
+     * Replica doesn't need to do anything for a checkpoint
+     */
+    return ORTE_SUCCESS;
+}
+
+
 /* this function is called by the event library and thus
  * can access information global to the state machine
  */
@@ -468,7 +532,9 @@ static void stdin_write_handler(int fd, short event, void *cbdata)
     orte_iof_write_event_t *wev = sink->wev;
     opal_list_item_t *item;
     orte_iof_write_output_t *output;
-    int num_written;
+    int num_written, total_written = 0;
+
+    ORTE_ACQUIRE_OBJECT(sink);
 
     OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,
                          "%s hnp:stdin:write:handler writing data to %d",
@@ -493,12 +559,7 @@ static void stdin_write_handler(int fd, short event, void *cbdata)
             OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
                                  "%s iof:hnp closing fd %d on write event due to zero bytes output",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), wev->fd));
-            OBJ_RELEASE(wev);
-            sink->wev = NULL;
-            /* just leave - we don't want to restart the
-             * read event!
-             */
-            return;
+            goto finish;
         }
         num_written = write(wev->fd, output->data, output->numbytes);
         OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,
@@ -512,9 +573,7 @@ static void stdin_write_handler(int fd, short event, void *cbdata)
                 /* leave the write event running so it will call us again
                  * when the fd is ready.
                  */
-                wev->pending = true;
-                opal_event_add(wev->ev, 0);
-                goto CHECK;
+                goto re_enter;
             }
             /* otherwise, something bad happened so all we can do is declare an
              * error and abort
@@ -523,9 +582,7 @@ static void stdin_write_handler(int fd, short event, void *cbdata)
             OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
                                  "%s iof:hnp closing fd %d on write event due to negative bytes written",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), wev->fd));
-            OBJ_RELEASE(wev);
-            sink->wev = NULL;
-            return;
+            goto finish;
         } else if (num_written < output->numbytes) {
             OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,
                                  "%s hnp:stdin:write:handler incomplete write %d - adjusting data",
@@ -537,14 +594,19 @@ static void stdin_write_handler(int fd, short event, void *cbdata)
             /* leave the write event running so it will call us again
              * when the fd is ready.
              */
-            wev->pending = true;
-            opal_event_add(wev->ev, 0);
-            goto CHECK;
+            goto re_enter;
         }
         OBJ_RELEASE(output);
-    }
 
-CHECK:
+        total_written += num_written;
+        if ((ORTE_IOF_SINK_BLOCKSIZE <= total_written) && wev->always_writable) {
+            goto re_enter;
+        }
+    }
+    goto check;
+  re_enter:
+    ORTE_IOF_SINK_ACTIVATE(wev);
+  check:
     if (NULL != mca_iof_hnp_component.stdinev &&
         !orte_abnormal_term_ordered &&
         !mca_iof_hnp_component.stdinev->active) {
@@ -564,42 +626,30 @@ CHECK:
             /* restart the read */
             OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,
                                  "restarting read event"));
-            mca_iof_hnp_component.stdinev->active = true;
-            opal_event_add(mca_iof_hnp_component.stdinev->ev, 0);
+            ORTE_IOF_READ_ACTIVATE(mca_iof_hnp_component.stdinev);
         }
     }
-}
-
-/**
- * Callback when peer is disconnected
- */
-
-static void
-orte_iof_hnp_exception_handler(orte_process_name_t* peer, orte_rml_exception_t reason)
-{
-#if 0
-    orte_iof_base_endpoint_t *endpoint;
-    opal_output_verbose(1, orte_iof_base_framework.framework_output,
-                        "iof svc exception handler! %s\n",
-                        ORTE_NAME_PRINT((orte_process_name_t*)peer));
-
-    /* If we detect an exception on the RML connection to a peer,
-     delete all of its subscriptions and publications.  Note that
-     exceptions can be detected during a normal RML shutdown; they
-     are recoverable events (no need to abort). */
-    orte_iof_hnp_sub_delete_all(peer);
-    orte_iof_hnp_pub_delete_all(peer);
-    opal_output_verbose(1, orte_iof_base_framework.framework_output, "deleted all pubs and subs\n");
-
-    /* Find any streams on any endpoints for this peer and close them */
-    while (NULL !=
-           (endpoint = orte_iof_base_endpoint_match(peer, ORTE_NS_CMP_ALL,
-                                                    ORTE_IOF_ANY))) {
-        orte_iof_base_endpoint_closed(endpoint);
-
-        /* Delete the endpoint that we just matched */
-        orte_iof_base_endpoint_delete(peer, ORTE_NS_CMP_ALL, ORTE_IOF_ANY);
+    if (sink->closed && 0 == opal_list_get_size(&wev->outputs)) {
+        /* the sink has already been closed and everything was written, time to release it */
+        OBJ_RELEASE(sink);
     }
-#endif
-    opal_output_verbose(1, orte_iof_base_framework.framework_output, "done with exception handler\n");
+    return;
+  finish:
+    OBJ_RELEASE(wev);
+    sink->wev = NULL;
+    return;
+}
+
+static int hnp_output(const orte_process_name_t* peer,
+                      orte_iof_tag_t source_tag,
+                      const char *msg)
+{
+    /* output this to our local output */
+    if (ORTE_IOF_STDOUT & source_tag || orte_xml_output) {
+        orte_iof_base_write_output(peer, source_tag, (const unsigned char*)msg, strlen(msg), orte_iof_base.iof_write_stdout->wev);
+    } else {
+        orte_iof_base_write_output(peer, source_tag, (const unsigned char*)msg, strlen(msg), orte_iof_base.iof_write_stderr->wev);
+    }
+
+    return ORTE_SUCCESS;
 }
