@@ -61,6 +61,11 @@ int mca_scoll_basic_alltoall(struct oshmem_group_t *group,
         return OSHMEM_ERR_BAD_PARAM;
     }
 
+    /* Do nothing on zero-length request */
+    if (OPAL_UNLIKELY(!nelems)) {
+        return OPAL_SUCCESS;
+    }
+
     if ((sst == 1) && (dst == 1)) {
         rc = a2a_alg_simple(group, target, source, nelems, element_size);
     } else {
@@ -72,14 +77,14 @@ int mca_scoll_basic_alltoall(struct oshmem_group_t *group,
        return rc;
     }
 
-    /* fence (which currently acts as quiet) is needed
-     * because scoll level barrier does not guarantee put completion
+    /* quiet is needed because scoll level barrier does not
+     * guarantee put completion
      */
-    MCA_SPML_CALL(fence());
+    MCA_SPML_CALL(quiet(oshmem_ctx_default));
 
     /* Wait for operation completion */
     SCOLL_VERBOSE(14, "[#%d] Wait for operation completion", group->my_pe);
-    rc = BARRIER_FUNC(group, pSync + 1, SCOLL_DEFAULT_ALG);
+    rc = BARRIER_FUNC(group, pSync, SCOLL_DEFAULT_ALG);
 
     /* Restore initial values */
     SCOLL_VERBOSE(12, "PE#%d Restore special synchronization array",
@@ -111,7 +116,7 @@ get_dst_pe(struct oshmem_group_t *group, int src_blk_idx, int dst_blk_idx, int *
     (*dst_pe_idx) = (dst_blk_idx + src_blk_idx) % group->proc_count;
 
     /* convert to the global pe */
-    return oshmem_proc_pe(group->proc_array[*dst_pe_idx]);
+    return oshmem_proc_pe_vpid(group, *dst_pe_idx);
 }
 
 static int a2as_alg_simple(struct oshmem_group_t *group,
@@ -138,7 +143,7 @@ static int a2as_alg_simple(struct oshmem_group_t *group,
 
         dst_pe = get_dst_pe(group, src_blk_idx, dst_blk_idx, &dst_pe_idx);
         for (elem_idx = 0; elem_idx < nelems; elem_idx++) {
-            rc = MCA_SPML_CALL(put(
+            rc = MCA_SPML_CALL(put(oshmem_ctx_default, 
                         get_stride_elem(target, tst, nelems, element_size,
                                         dst_blk_idx, elem_idx),
                         element_size,
@@ -178,7 +183,7 @@ static int a2a_alg_simple(struct oshmem_group_t *group,
     for (src_blk_idx = 0; src_blk_idx < group->proc_count; src_blk_idx++) {
 
         dst_pe = get_dst_pe(group, src_blk_idx, dst_blk_idx, &dst_pe_idx);
-        rc = MCA_SPML_CALL(put(dst_blk,
+        rc = MCA_SPML_CALL(put(oshmem_ctx_default, dst_blk,
                                 nelems * element_size,
                                 get_stride_elem(source, 1, nelems,
                                                 element_size, dst_pe_idx, 0),
