@@ -1,7 +1,9 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -13,7 +15,7 @@
 
 #include <string.h>
 
-#include "ompi/mpi/fortran/base/strings.h"
+#include "ompi/mpi/fortran/base/fortran_base_strings.h"
 
 #include "oshmem/shmem/fortran/bindings.h"
 #include "oshmem/include/shmem.h"
